# Copyright 2017, 2018 CA Technologies. All Rights Reserved.

import re
from vcsconn.utils.eif_exception import ConfigurationError, OperationalError

class AgicenTransformer:
    def __init__(self, config):
        self.config = config

    def validate(self):
        # make sure that for each config key there is only 1 value
        pass

    def getTransformArgument(self,  transform):
        pattern = r'\((.*?)\)'
        result = re.findall(pattern, transform, re.IGNORECASE)
        return [x for x in result][0]

    def getUserMapping(self, file_name):
        user_map = {}
        file_path = "config/{}".format(file_name)
        try:
            with open(file_path, 'r') as f:
                _ = next(f)    # burn off the first line which is actually a header line
                for line in f:
                    vcs_user, ac_username = line.strip().rsplit(' ', 1)

                    user_map[vcs_user] = ac_username
        except:
            problem = "Unable to open file: %s" % file_path
            raise ConfigurationError(problem)
        return user_map

    def loadTransforms(self):
        user_map = {}
        valid_transform_attributes = ['Author', 'Message']
        author_transform = self.config['Author']
        if author_transform.startswith('FileBasedUserNameLookup'):
            file_name = self.getTransformArgument(author_transform)
            user_map = self.getUserMapping(file_name)
            author_transform = 'FileBasedUserNameLookup'
        elif author_transform.startswith('UserNameDomainAugmentLookup'):
            domain_name = self.getTransformArgument(author_transform)
            user_map = {'DOMAIN': domain_name}
            author_transform = 'UserNameDomainAugmentLookup'
        if user_map:
            self.config['Author'] =  (author_transform, user_map)
        else:
            self.config['Author'] =  (author_transform,)

    def findACUserByFileBasedLookup(self, user_map, user):
        author = None
        found = [{gh_user: ac_user} for gh_user, ac_user in user_map.items() if gh_user == user]
        if found:
            author = found[0][user]
        return author

    def getAuthor(self, missing_changeset, transformer):
        author = None
        if len(transformer) > 1:
            if transformer[0] == 'FileBasedUserNameLookup':
                author = self.findACUserByFileBasedLookup(transformer[1], missing_changeset.committer)

            elif transformer[0] == 'UserNameDomainAugmentLookup':
                author = missing_changeset.committer + '@' + transformer[1]['DOMAIN']
        return author

    def transform(self, transform_field, changeset):
        if transform_field not in self.config:
            return getattr(changeset, transform_field)
        if transform_field == 'Author':
            return self.getAuthor(changeset, self.config['Author'])
        return None
