# Copyright 2017, 2018 CA Technologies. All Rights Reserved.

from vcsconn.utils.chronuti import TimeStamp, ISO_FORMAT

class BitbucketChangeset:

    def __init__(self, bb_changeset):
        self.node             = None
        self.ident            = None
        self.rev_num          = None
        self.committer        = None
        self.committer_id     = None
        self.committer_name   = None
        self.committer_email  = None
        self.commit_timestamp = None
        self.message          = None
        self.path             = ''
        self.branch           = ''

        self.cloud = True if 'hash' in bb_changeset else False
        self._extractFields(bb_changeset)
        self.rev_num = self.ident

    def setPath(self, base_url):
        self.path = "%s/commits/%s" % (base_url, self.ident)

    def _extractFields(self, bb_changeset):
        if self.cloud:
            self.ident            = bb_changeset['hash']
            self.message          = bb_changeset['message']
            self.committer        = bb_changeset['author']['raw'].split(' <')[0]
            self.committer_name   = self.committer
            self.committer_email  = bb_changeset['author']['raw'].split(' <')[1][:-1]
            self.commit_timestamp = TimeStamp.fromFormattedString(bb_changeset['date'][:-6] + 'Z',ISO_FORMAT)

        else:
            self.ident            = bb_changeset['id']
            self.message          = bb_changeset['message']
            self.committer        = ""  # this is to protect against unexpected absence of author -> displayName|name items in JSON
            # assign the committer to the bb_changeset['author']['displayName'] if such JSON exists...
            if 'displayName' in bb_changeset['author']:
                self.committer = bb_changeset['author']['displayName']
            # but if bb_changeset['author']['name'] is present use that for the assignment
            elif 'name' in bb_changeset['author']:
                self.committer = bb_changeset['author']['name']
            self.committer_name   = self.committer
            self.committer_email  = bb_changeset['author']['emailAddress']
            self.commit_timestamp = TimeStamp.fromEpochSeconds(bb_changeset['authorTimestamp'] / 1000)


    def __lt__(self, othr):
        return self.commit_timestamp < othr.commit_timestamp

    def details(self):
        ident      = "Ident: %s"        %  self.ident
        committer  = "Comitter: %s"     %  self.committer
        committed  = "Committed: %s"    %  self.commit_timestamp.readable
        message    = "Message: %s"      %  self.message
        elements = [ident, committer, committed, message]
        elements.append("")
        pill = "\n".join(elements)
        return pill


######################################################################################################

example_json_response_from_Bitbucket = """\

            You'll get a  dictionary that looks like for Bitbucket Server:
        {
         'id'           : '47fbd90808cb19052e844fe95ab4548a426ab385',
         'displayId'    : '47fbd90808c',
         'committer'       : {
                            'name'        : 'jpkoleself.ca.com',
                            'emailAddress': 'jpkoleself.ca.com',
                            'id'          : 2,
                            'displayName' : 'jpkoleself.ca.com',
                            'active'      : True,
                            'slug'        : 'jpkole_ca.com',
                            'type'        : 'NORMAL',
                            'links'       : { 'self' => [{'href''=>'http://ec2-54-211-2-37:7990/users/jpkole_ca.com'}] }
                           },
         'committerTimestamp': 1492546931000,
         'message'      : 'commit message - Tue Apr 18 14:22:11 MDT 2017',
         'parents'      :  [{'id'        : '797de964bd0a418e98d5aac7c957e863f18e74c5',
                             'displayId' : '797de964bd0'}]
        }

        and something like this for Bitbucket Cloud:


{
"date": "2017-11-07T20:11:14+00:00",
"message": "agua-fountain.pdl branch test for DE8",
"type": "commit",
"type": "repository",
"name": "evinrude",
"full_name": "TechServicesBitbucket/evinrude",
"uuid": "{9c340562-a135-4e2e-9762-52cfd6ac68e0}",
"hash": "c3430af59a1307ebe91ffa68075ab0f09d57ad67",
"repository": {
"links": {
"self": {
"href": "https://api.bitbucket.org/2.0/repositories/TechServicesBitbucket/evinrude"
},
"html": {
"href": "https://bitbucket.org/TechServicesBitbucket/evinrude"
},
"avatar": {
"href": "https://bitbucket.org/TechServicesBitbucket/evinrude/avatar/32/"
}
},
},
"links": {
"self": {
"href": "https://api.bitbucket.org/2.0/repositories/TechServicesBitbucket/evinrude/commit/c3430af59a1307ebe91ffa68075ab0f09d57ad67"
},
"comments": {
"href": "https://api.bitbucket.org/2.0/repositories/TechServicesBitbucket/evinrude/commit/c3430af59a1307ebe91ffa68075ab0f09d57ad67/comments"
},
"html": {
"href": "https://bitbucket.org/TechServicesBitbucket/evinrude/commits/c3430af59a1307ebe91ffa68075ab0f09d57ad67"
},
},
"author": {},
"parents": [
{
"hash": "16e0c33deec94325a952dae8d06f971e4eef8db1",
"type": "commit",
"links": {
"self": {
"href": "https://api.bitbucket.org/2.0/repositories/TechServicesBitbucket/evinrude/commit/16e0c33deec94325a952dae8d06f971e4eef8db1"
},
}
}
],
}
        """