# Copyright 2017, 2018 CA Technologies. All Rights Reserved.

from vcsconn.utils.chronuti import TimeStamp, ISO_FORMAT

class BitbucketPullRequest:

    def __init__(self, pull_req):
        self.number = pull_req['id']
        self.title  = pull_req['title']
        self.state  = pull_req['state']
        self.description = pull_req['description']

        self.creation_timestamp = None
        self.description = None
        self.author      = None
        self.commits_url = None
        self.bb_helper   = None

        self.cloud = True if 'source' in pull_req else False
        self._extractFields(pull_req)

    def _extractFields(self, pull_req):
        if self.cloud:
            self.ident   = pull_req['source']['commit']['hash']
            self.url     = pull_req['links']['self']['href']
            created      = pull_req['created_on']
            self.creation_timestamp = TimeStamp.fromFormattedString(created, ISO_FORMAT)
            self.author        = pull_req['author']['username']
            # self.author_name = pull_req['author']['display_name']
            self.commits_url   = pull_req['links']['commits']['href']
        else:
            self.ident = pull_req['fromRef']['latestCommit']
            self.url   = pull_req['links']['self'][0]['href']
            created = pull_req['createdDate']/1000
            self.creation_timestamp = TimeStamp.fromEpochSeconds(created)
            self.author = pull_req['author']['user']['slug']
            self.commits_url = self.constructCommitsUrlForServer()


    def constructCommitsUrlForServer(self):
        # use self.url
        # http://koljo03-s4576.ca.com:7990/projects/GUN1/repos/rocket-man/pull-requests/1"
        #  to construct an api url that follows this format:
        # http://koljo03-s4576.ca.com:7990/rest/api/1.0/projects/GUN1/repos/rocket-man/pull-requests/1/commits
        separator = '/projects'
        server, endpoint = self.url.split(separator)
        commits_url = "%s/rest/api/1.0%s%s/commits" % (server, separator, endpoint)
        return commits_url


    def __lt__(self, othr):
        return self.creation_timestamp < othr.creation_timestamp

    def details(self):
        ident      = "Ident SHA1: %s" % self.ident
        number     = "Number: %s"     % self.number
        created    = "Created: %s"    % self.creation_timestamp.readable
        title      = "Title: %s"      % self.title
        state      = "State: %s"      % self.state
        #desc       = "Description: %s"  % self.description[:20]
        #author     = "Author: %s"        % self.author
        elements = [ident, number, created, title, state]

        elements.insert(0, '')
        elements.append('')
        pill = "\n    ".join(elements)
        return pill

    def __str__(self):
        return "%s  %3d  %s  State: %-8.8s   %s" % (
            self.ident, int(self.number), self.creation_timestamp.readable, self.state, self.title)

    def extractCommitMessages(self):
        commits_url  = "%s?pagelen=20" % self.commits_url
        pr_related_commits = self.bb_helper.getCommits(commits_url)
        commit_messages = [commit['message'] for commit in pr_related_commits]
        commit_messages.append(self.title)
        return commit_messages

##############################################################################################

pull_request_json_response_example = \
"""
            Here is a cloud json.
            You'll get a  dictionary that looks like. If there is no next page, "next" key will be missing
{
    "next": "https://api.bitbucket.org/2.0/repositories/LoriTeam1/humerus/pullrequests?pagelen=2&state=ALL&page=2"
    "page": 1,
    "pagelen": 10,
    "size": 1,
    "values": [
        {
            "author": {"display_name": "Lori Bell"..."username": "loribell01"},
            "created_on": "2017-11-07T15:38:44.959390+00:00",
            "description": "",
            "destination": {
                "branch": {
                    "name": "twister"
                },
                "commit": {
                    "hash": "7f1a0c34f0e5",
                    "links": {
                        "self": {
                            "href": "https://api.bitbucket.org/2.0/repositories/loriteam1/humerus/commit/7f1a0c34f0e5"
                        }
                    }
                },
                "repository": {
                    "full_name": "loriteam1/humerus",
                    "name": "humerus"
                }
            },
            "id": 1,
            ...
            "merge_commit": null,
            "reason": "",
            "source": {
                "branch": {
                    "name": "master"
                },
                "commit": {
                    "hash": "afa04dd5d9e3",
                    "links": {
                        "self": {
                            "href": "https://api.bitbucket.org/2.0/repositories/loriteam1/humerus/commit/afa04dd5d9e3"
                        }
                    }
                },
                "repository": {
                    "full_name": "loriteam1/humerus",
                    "name": "humerus",
                }
            },
            "state": "OPEN",
            "title": "README.md edited on twister branch US127",
            "type": "pullrequest",
            "updated_on": "2017-11-07T15:38:45.182888+00:00"
        }
    ]
}
"""
