# Copyright 2017, 2018 CA Technologies. All Rights Reserved.

from vcsconn.utils.chronuti import TimeStamp, ISO_FORMAT

class TFSGitPullRequest:
    API_VERSION = "3.0"

    def __init__(self, pull_req):
        self.number = pull_req['pullRequestId']
        self.ident  = pull_req['lastMergeSourceCommit']['commitId']
        self.title  = pull_req['title']
        self.state  = pull_req['status']
        self.description = pull_req.get('description','')
        self.creation_timestamp = TimeStamp.fromFormattedString(pull_req['creationDate'][:-5] + 'Z', ISO_FORMAT)
        self.url = self.constructPath(pull_req['url'], pull_req['repository']['project'], pull_req['repository']['name'])
        self.commits_url   = "%s/commits" % pull_req['url']
        self.tfsgit_helper = None

    def constructPath(self, url, project, repo):
        base_url = url.split('/_apis/git/repositories/')[0]
        url = "%s/%s/_git/%s/pullrequest/%s" %(base_url, project['name'], repo, self.number)
        # collection, project, "_git", repo, "pullrequest", prnumber
        return url

    def __lt__(self, othr):
        return self.creation_timestamp < othr.creation_timestamp

    def details(self):
        ident      = "Ident SHA1: %s" % self.ident
        number     = "Number: %s"     % self.number
        created    = "Created: %s"    % self.creation_timestamp.readable
        title      = "Title: %s"      % self.title
        state      = "State: %s"      % self.state
        elements = [ident, number, created, title, state]

        elements.insert(0, '')
        elements.append('')
        pill = "\n    ".join(elements)
        return pill

    def __str__(self):
        return "%s  %3d  %s  State: %-8.8s   %s" % (
            self.ident, int(self.number), self.creation_timestamp.readable, self.state, self.title)

    def extractCommitMessages(self):
        """
            This method is in some sense a "deferred" method.  It is only called from the outside
            in the event that commit messages are needed.  In the event for this PullRequest the
            commit messages aren't needed then we don't incur the expense of an external request.
        """
        commits_url  = "%s?api-version=%s" % (self.commits_url, TFSGitPullRequest.API_VERSION)
        pr_related_commits = self.tfsgit_helper.getFromTFSGit(commits_url)
        commit_messages = [commit['comment'] for commit in pr_related_commits['value']]
        commit_messages.append(self.title)
        return commit_messages