# Copyright 2017, 2018 CA Technologies. All Rights Reserved.

import inspect
import requests
from vcsconn.utils.eif_exception import ConfigurationError, OperationalError
from vcsconn.utils.klog       import getCaller
from vcsconn.vsts_changeset   import VSTSChangeset
from vcsconn.vsts_pullrequest import VSTSPullRequest


class VSTSHelper:

    STD_PAGESIZE   =  100
    REPOS_PAGESIZE = 1000

    def __init__(self, username, token, logger, proxy):
        self.username = username
        self.token    = token
        self.log      = logger
        self.http_proxy = proxy

        if self.username and self.token:
            self.auth = (self.username, self.token)

    def getFromVSTS(self, uri):  # internal helper method that makes requests to Bitbucket Server REST API
        result = None
        stack_item = inspect.stack()[1]  # assume caller of interest is stack_levels down in the call stack
        frame, filename, line_no, code_unit = stack_item[:4]
        caller = getCaller(frame)
        self.log.debug("VSTS request URL on behalf of %s: %s" % (caller, uri))
        try:
            headers = {'Content-Type': 'application/json'}
            response = requests.get(uri, auth=self.auth, proxies=self.http_proxy, headers=headers)
            result = response.json()
        except Exception as exc:
            actual_error = exc.args[0]
            problem = "Unable to retrieve information from VSTS. %s" % actual_error
            raise OperationalError(problem)

        if 'error' in result:
            msg = result['error']['message']
            raise OperationalError(msg)
        if 'errors' in result:
            msg = result['errors'][0]['message']
            raise OperationalError(msg)

        return result

    def getAllRepos(self, repos_url):
        repos_url = repos_url + '&limit=%s' % VSTSHelper.REPOS_PAGESIZE
        # below is an intentionally invalid url:  (for dev/test purposes only)
        #repos_url =  repos_url +'/notok' + '?limit=%s' % BitbucketHelper.REPOS_PAGESIZE
        try:
            result = self.getFromVSTS(repos_url)
        except Exception as ex:
            ex.report()
            return []
        return result['value']

    def getQualifiedCommits(self, url, ref_time):
        commits = []
        try:
            url = "%s&fromDate=%s" %(url, ref_time.readable)
            results = self.getFromVSTS(url)
        except Exception as ex:
            ex.report()
            return ([],'')

        for commit in results['value']:
            commits.append(VSTSChangeset(commit))

        return commits

    def getQualifiedPullRequests(self, repo_url, ref_time):
        pull_requests = []
        try:
            pr_url = "%s&fromDate=%s" % (repo_url, ref_time.readable)
            results = self.getFromVSTS(pr_url)
        except Exception as ex:
            ex.report()
            return []
        for pr in results['value']:
            pull_request = VSTSPullRequest(pr)
            pull_request.vsts_helper = self
            pull_requests.append(pull_request)
        return pull_requests


    def getCommits(self, commits_url):
        try:
            result = self.getFromVSTS(commits_url)
        except Exception as ex:
            ex.report()
            return []
        return result['value']